==> v0.8.2.tar.gz: neovim/test/functional/legacy/prompt_buffer_spec.lua <==
local helpers = require('test.functional.helpers')(after_each)
local Screen = require('test.functional.ui.screen')
local feed = helpers.feed
local source = helpers.source
local clear = helpers.clear
local feed_command = helpers.feed_command
local poke_eventloop = helpers.poke_eventloop
local meths = helpers.meths
local eq = helpers.eq

describe('prompt buffer', function()
  local screen

  before_each(function()
    clear()
    screen = Screen.new(25, 10)
    screen:attach()
    source([[
      func TextEntered(text)
        if a:text == "exit"
          set nomodified
          stopinsert
          close
        else
          call append(line("$") - 1, 'Command: "' . a:text . '"')
          set nomodified
          call timer_start(20, {id -> TimerFunc(a:text)})
        endif
      endfunc

      func TimerFunc(text)
        call append(line("$") - 1, 'Result: "' . a:text .'"')
      endfunc

      func SwitchWindows()
        call timer_start(0, {-> execute("wincmd p", "")})
      endfunc
    ]])
    feed_command("set noshowmode | set laststatus=0")
    feed_command("call setline(1, 'other buffer')")
    feed_command("new")
    feed_command("set buftype=prompt")
    feed_command("call prompt_setcallback(bufnr(''), function('TextEntered'))")
    feed_command("eval bufnr('')->prompt_setprompt('cmd: ')")
  end)

  after_each(function()
    screen:detach()
  end)

  it('works', function()
    screen:expect([[
      ^                         |
      ~                        |
      ~                        |
      ~                        |
      [Prompt]                 |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]])
    feed("i")
    feed("hello\n")
    screen:expect([[
      cmd: hello               |
      Command: "hello"         |
      Result: "hello"          |
      cmd: ^                    |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]])
    feed("exit\n")
    screen:expect([[
      ^other buffer             |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]])
  end)

  it('editing', function()
    screen:expect([[
      ^                         |
      ~                        |
      ~                        |
      ~                        |
      [Prompt]                 |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]])
    feed("i")
    feed("hello<BS><BS>")
    screen:expect([[
      cmd: hel^                 |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]])
    feed("<Left><Left><Left><BS>-")
    screen:expect([[
      cmd: -^hel                |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]])
    feed("<C-O>lz")
    screen:expect([[
      cmd: -hz^el               |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]])
    feed("<End>x")
    screen:expect([[
      cmd: -hzelx^              |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]])
    feed("<C-U>exit\n")
    screen:expect([[
      ^other buffer             |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]])
  end)

  it('switch windows', function()
    feed_command("set showmode")
    feed("i")
    screen:expect([[
      cmd: ^                    |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
      -- INSERT --             |
    ]])
    feed("<C-O>:call SwitchWindows()<CR>")
    screen:expect{grid=[[
      cmd:                     |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      ^other buffer             |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]]}
    feed("<C-O>:call SwitchWindows()<CR>")
    screen:expect([[
      cmd: ^                    |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
      -- INSERT --             |
    ]])
    feed("<Esc>")
    screen:expect([[
      cmd:^                     |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]])
  end)

  it('keeps insert mode after aucmd_restbuf in callback', function()
    source [[
      let s:buf = nvim_create_buf(1, 1)
      call timer_start(0, {-> nvim_buf_set_lines(s:buf, -1, -1, 0, ['walrus'])})
      startinsert
    ]]
    poke_eventloop()
    eq({ mode = "i", blocking = false }, meths.get_mode())
  end)
end)

==> v0.9.0.tar.gz: neovim/test/functional/legacy/prompt_buffer_spec.lua <==
local helpers = require('test.functional.helpers')(after_each)
local Screen = require('test.functional.ui.screen')
local feed = helpers.feed
local source = helpers.source
local clear = helpers.clear
local command = helpers.command
local poke_eventloop = helpers.poke_eventloop
local meths = helpers.meths
local eq = helpers.eq
local neq = helpers.neq

describe('prompt buffer', function()
  local screen

  before_each(function()
    clear()
    screen = Screen.new(25, 10)
    screen:attach()
    command('set laststatus=0 nohidden')
  end)

  local function source_script()
    source([[
      func TextEntered(text)
        if a:text == "exit"
          " Reset &modified to allow the buffer to be closed.
          set nomodified
          stopinsert
          close
        else
          " Add the output above the current prompt.
          call append(line("$") - 1, 'Command: "' . a:text . '"')
          " Reset &modified to allow the buffer to be closed.
          set nomodified
          call timer_start(20, {id -> TimerFunc(a:text)})
        endif
      endfunc

      func TimerFunc(text)
        " Add the output above the current prompt.
        call append(line("$") - 1, 'Result: "' . a:text .'"')
        " Reset &modified to allow the buffer to be closed.
        set nomodified
      endfunc

      func SwitchWindows()
        call timer_start(0, {-> execute("wincmd p", "")})
      endfunc

      call setline(1, "other buffer")
      set nomodified
      new
      set buftype=prompt
      call prompt_setcallback(bufnr(''), function("TextEntered"))
      eval bufnr("")->prompt_setprompt("cmd: ")
      startinsert
    ]])
    screen:expect([[
      cmd: ^                    |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
      -- INSERT --             |
    ]])
  end

  after_each(function()
    screen:detach()
  end)

  -- oldtest: Test_prompt_basic()
  it('works', function()
    source_script()
    feed("hello\n")
    screen:expect([[
      cmd: hello               |
      Command: "hello"         |
      Result: "hello"          |
      cmd: ^                    |
      [Prompt]                 |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
      -- INSERT --             |
    ]])
    feed("exit\n")
    screen:expect([[
      ^other buffer             |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]])
  end)

  -- oldtest: Test_prompt_editing()
  it('editing', function()
    source_script()
    feed("hello<BS><BS>")
    screen:expect([[
      cmd: hel^                 |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
      -- INSERT --             |
    ]])
    feed("<Left><Left><Left><BS>-")
    screen:expect([[
      cmd: -^hel                |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
      -- INSERT --             |
    ]])
    feed("<C-O>lz")
    screen:expect([[
      cmd: -hz^el               |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
      -- INSERT --             |
    ]])
    feed("<End>x")
    screen:expect([[
      cmd: -hzelx^              |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
      -- INSERT --             |
    ]])
    feed("<C-U>exit\n")
    screen:expect([[
      ^other buffer             |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]])
  end)

  -- oldtest: Test_prompt_switch_windows()
  it('switch windows', function()
    source_script()
    feed("<C-O>:call SwitchWindows()<CR>")
    screen:expect{grid=[[
      cmd:                     |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      ^other buffer             |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]]}
    feed("<C-O>:call SwitchWindows()<CR>")
    screen:expect([[
      cmd: ^                    |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
      -- INSERT --             |
    ]])
    feed("<Esc>")
    screen:expect([[
      cmd:^                     |
      ~                        |
      ~                        |
      ~                        |
      [Prompt] [+]             |
      other buffer             |
      ~                        |
      ~                        |
      ~                        |
                               |
    ]])
  end)

  -- oldtest: Test_prompt_while_writing_to_hidden_buffer()
  it('keeps insert mode after aucmd_restbuf in callback', function()
    source_script()
    source [[
      let s:buf = nvim_create_buf(1, 1)
      call timer_start(0, {-> nvim_buf_set_lines(s:buf, -1, -1, 0, ['walrus'])})
    ]]
    poke_eventloop()
    eq({ mode = 'i', blocking = false }, meths.get_mode())
  end)

  -- oldtest: Test_prompt_appending_while_hidden()
  it('accessing hidden prompt buffer does not start insert mode', function()
    local prev_win = meths.get_current_win()
    source([[
      new prompt
      set buftype=prompt
      set bufhidden=hide

      func s:TextEntered(text)
          if a:text == 'exit'
              close
          endif
          let g:entered = a:text
      endfunc
      call prompt_setcallback(bufnr(), function('s:TextEntered'))

      func DoAppend()
        call appendbufline('prompt', '$', 'Test')
      endfunc
    ]])
    feed('asomething<CR>')
    eq('something', meths.get_var('entered'))
    neq(prev_win, meths.get_current_win())
    feed('exit<CR>')
    eq(prev_win, meths.get_current_win())
    command('call DoAppend()')
    eq({ mode = 'n', blocking = false }, meths.get_mode())
  end)
end)
